# QTRES - Process a single TRES image from a raw data file to a spectrum
# January 23, 2008
# By Doug Mink, SAO Telescope Data Center

procedure qtres (image)

string	image=""	{prompt="Image to process"}
string	imdate=""	{prompt="Date for image"}
string	instdir=""	{prompt="Directory containing date directory"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$"     {prompt="Directory for normalization file"}
bool	fixpreamp=yes	{prompt="Fix preamp time constant (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$"	{prompt="Directory for calibration lamp file"}
int	plotorder=1	{prompt="Echelle order to plot (No plot if zero"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	ims1, ims2, ims3, imsx, imin, impath, cdir, calref, newref, newfile
string	imroot, imd, croot, tsize, tbin
int	pl
bool	vb, dt, sv, f1, f2, f3

	vb = verbose
	dt = deltemp
	sv = saveold
	pl = plotorder
	imd = imdate
	imin = image
	cdir = compdir
	croot = comproot

	if (imin == "") {
	    print ("QTRES: Reduce a single TRES image in a different directory")
	    print ("Usage: qtres image with both amps to be corrected")
	    return
	    }

# Copy image to current directory
	if (instdir != "") {
	    fixdir (instdir)
	    impath = fixdir.newdir
	    if (imd != "")
		impath = impath // imd // "/" // imin
	    else
		impath = impath // imin
	    if (!access (imin))
		copy (impath, imin)
	    }

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpreamp = fixpreamp
	tproc.flatten = yes
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.deltemp = dt
	tproc.saveold = sv
	tproc.verbose = vb
	tproc (imin)

# Extract spectra
	textract.aproot = ""
	textract.apdir = ""
	textract.recenter = recenter
	textract.deltemp = dt
	textract.verbose = vb
	textract (imin)

# Figure out aperture size, binning, and fibers present
	trsfib (imin, verbose-, debug-)
	tsize = trsfib.apsize
	tsize = ""
	tbin = trsfib.binning
	if (trsfib.fib1 != "OFF")
	    f1 = yes
	else
	    f1 = no
#	if (trsfib.fib2 != "OFF")
#	    f2 = yes
#	else
	    f2 = no
	if (trsfib.fib3 != "OFF")
	    f3 = yes
	else
	    f3 = no
	fileroot (imin)
	imroot = fileroot.rootname

# Add wavelength solution to spectra
	dispcor.linearize = no
	dispcor.database = "database"
	dispcor.table = ""
	dispcor.w1 = INDEF
	dispcor.w2 = INDEF
	dispcor.dw = INDEF
	dispcor.nw = INDEF
	dispcor.log = no
	dispcor.flux = yes
	dispcor.blank = 0.
	dispcor.samedisp = no
	dispcor.global = no
	dispcor.ignoreaps = no
	dispcor.confirm = no
	dispcor.listonly = no
	dispcor.verbose = vb
	dispcor.logfile = ""
	if (f1) {
	    ims1 = imroot // "_fib1.ec.fits"
	    calref = cdir//croot//tsize//tbin//"1"//".ec.fits"
	    newref = croot//tsize//tbin//"1"//".ec.fits"
	    if (!access (newref)) {
		copy (calref, newref)
		}
	    calref = cdir//"database/ec"//croot//tsize//tbin//"1"//".ec"
	    newref = "database/ec"//croot//tsize//tbin//"1"//".ec"
	    if (!access (newref)) {
		copy (calref, newref)
		}
	    calref = croot//tsize//tbin//"1"//".ec"
	    hedit (ims1, "REFSPEC1", calref, add+, verify-, show=vb, update+)
	    imsx = imroot // "_fib1_disp.ec.fits"
	    dispcor (ims1, imsx, linearize=no)
	    if (sv) {
		if (!access ("Nodisp/")) {
		    mkdir ("Nodisp")
		    }
		newfile = "Nodisp/" // ims1
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (ims1, "Nodisp")
		if (vb) {
		    print ("QTRES: Original ", ims1, " => Nodisp/")
		    }
		}
	    else {
		imdelete (ims1)
		}
	    rename (imsx, ims1)
	    }
	if (f2) {
	    ims2 = imroot // "_fib2.ec.fits"
	    calref = cdir//croot//tsize//tbin//"2"//".ec.fits"
	    newref = croot//tsize//tbin//"2"//".ec.fits"
	    if (!access (newref)) {
		copy (calref, newref)
		}
	    calref = cdir//"database/ec"//croot//tsize//tbin//"2"//".ec"
	    newref = "database/ec"//croot//tsize//tbin//"2"//".ec"
	    if (!access (newref)) {
		copy (calref, newref)
		}
	    calref = croot//tsize//tbin//"2"//".ec"
	    hedit (ims2, "REFSPEC1", calref, verify-, show=vb, add+, update+)
	    imsx = imroot // "_fib2_disp.ec.fits"
	    dispcor (ims2, imsx, linearize=no)
	    if (sv) {
		if (!access ("Nodisp/")) {
		    mkdir ("Nodisp")
		    }
		newfile = "Nodisp/" // ims1
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (ims2, "Nodisp")
		if (vb) {
		    print ("HCAL: Original ", ims2, " => Nodisp/")
		    }
		}
	    else {
		imdelete (ims2)
		}
	    rename (imsx, ims2)
	    }
	if (f3) {
	    ims3 = imroot // "_fib3.ec.fits"
	    calref = cdir//croot//tsize//tbin//"2"//".ec.fits"
	    newref = croot//tsize//tbin//"2"//".ec.fits"
	    if (!access (newref)) {
		copy (calref, newref)
		}
	    calref = cdir//"database/ec"//croot//tsize//tbin//"2"//".ec"
	    newref = "database/ec"//croot//tsize//tbin//"2"//".ec"
	    if (!access (newref)) {
		copy (calref, newref)
		}
	    calref = croot//tsize//tbin//"2"//".ec"
	    hedit (ims1, "REFSPEC1", calref, verify-, add+, show=vb, update+)
	    imsx = imroot // "_fib2_disp.ec.fits"
	    dispcor (ims3, imsx, linearize=no)
	    if (sv) {
		if (!access ("Nodisp/")) {
		    mkdir ("Nodisp")
		    }
		newfile = "Nodisp/" // ims1
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (ims3, "Nodisp")
		if (vb) {
		    print ("HCAL: Original ", ims3, " => Nodisp/")
		    }
		}
	    else {
		imdelete (ims3)
		}
	    rename (imsx, ims3)
	    }

# Plot spectra if desired
	if (pl > 0) {
	    if (f1)
		splot (ims1, pl)
	    if (f2)
		splot (ims2, pl)
	    if (f3)
		splot (ims3, pl)
	    }
end

# Oct 12 2007	New task for quick look
# Dec 11 2007	Remove old declaration of instdir from before it was a parameter

# Jan 16 2008	Add fixpreamp parameter
# Jan 17 2008	Use tproc instead of tpreproc and tflatten
# Jan 23 2008	Add plotorder parameter to plot spectrum at end
# Jan 23 2008	Add recenter to make recentering during extraction optional
# Jan 23 2008	Add comproot parameter and wavelength calibration
