# TCOSMIC - Remove cosmic rays from a list of images
# November 14, 2007
# By Doug Mink, SAO Telescope Data Center
# CL and SSP implementation of Nelson Caldwell's Perl scripts

procedure tcosmic (images)

string	images=""	{prompt="Images to combine"}
real	threshold=10.0	{prompt="Threshold for cosmic ray difference"}
int	xbox=3		{prompt="Horizontal dimension of statistics box"}
int	ybox=20		{prompt="Vertical dimension of statistics box"}
string	fill="image"	{prompt="Fill bad pixels (image | median | fixpix)"}
bool	recompute=no	{prompt="Recompute median if already there (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool saveold=yes	{prompt="Save unprocessed image files in Uncosmic/ (y or n)"}
bool	savebp=yes	{prompt="Save generated bad pixel files in Uncosmic/ (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	medfile, limfile, templist, imin1, imin2, imbp1, imbp2, imr
int	ifile, nfiles, lroot, ipass
string	tchar, imagelist, history, dstring
string	outroot, imroot, fn1, fn2
file	listfile, imfile
real	medmed, high, oldnsig
bool	dellist, vb, recomp, done, movebp, moveold

	vb = verbose
	dellist = deltemp
	movebp = savebp
	moveold = saveold
	imagelist = images
	recomp = recompute

# Make directory in which to save preampfixed multi-extension files
	if (!access ("Uncosmic")) {
	    mkdir ("Uncosmic")
	    }

# Load needed packages if necessary
	if (!defpac ("imred")) {
	    imred
	    }
	if (!defpac ("proto")) {
	    proto
	    }

#  Make temporary list of input files
	templist = mktemp ("tempcosmic") // ".list"
	files (imagelist, >templist)

# Count files, get spectrum dimension, and initialize output root name
	nfiles = 0
	done = yes
	list = templist
	while (fscan (list,imin1) != EOF) {
	    nfiles = nfiles + 1
	    if (nfiles == 1) {
		smatch (imin1, ".fits")
		if (smatch.found) {
		    outroot = substr (imin1, 6, 100)
		    }
		else {
		    outroot = substr (imin1, 6, 100) // ".fits"
		    }
		fn1 = substr (imin1,1,4)
		}
	    imgetp.default = ""
	    imgetp (imin1, "TCOSMIC")
	    if (imgetp.value == "") {
		done = no
		}
	    fn2 = substr (imin1,1,4)
	    }
	if (done == yes) {
	    print ("TCOSMIC: Cosmic rays already removed from this pointing")
	    return
	    }
	if (nfiles == 0) {
	    print ("TCOSMIC: No files to process")
	    return
	    }
	else if (nfiles == 1) {
	    print ("TCOSMIC: Cannot process only one file")
	    return
	    }
	else if (vb) {
	    print (nfiles," files to process")
	    }

#  Make variance of median of input images
	medfile = "med_" // fn1 // "_" // fn2 // outroot
	if (recompute && access (medfile)) {
	    delete (medfile)
	    }
	if (!access (medfile)) {
	    tcosmed (imagelist, medfile=medfile, nfiles=nfiles, verbose=vb)
	    }
	limfile = "lim_" // fn1 // "_" // fn2 // outroot

# Delete existing limit file if it did not use the same nsigma
	if (recompute && access (limfile)) {
	    delete (limfile)
	    }
	if (access (limfile)) {
	    imgetp.default = "0.0"
	    imgetp (limfile, "TCOSNSIG")
	    oldnsig = real (imgetp.value)
	    if (oldnsig != threshold) {
		delete (limfile)
		}
	    }
	if (!access (limfile)) {
	    tcoslim.threshold = threshold
	    tcoslim (medfile, limfile, verbose=vb)
	    }

# Set parameters if replacing bad pixels using fixpix
	if (fill == "fixpix") {
	    tcosbad.limfile = limfile
	    tcosbad.verbose = vb
	    tcosbad.xbox = xbox
	    tcosbad.ybox = ybox
	    tcosbad.proc2 = yes
	    imreplace.lower = 2.0
	    imreplace.upper = 4.0
	    imreplace.radius = 1.5
	    fixpix.linterp = "INDEF"
	    fixpix.cinterp = "INDEF"
	    fixpix.verbose = vb
	    fixpix.pixels = no
	    }

# Set parameters if replacing bad pixels from second image or median
	else {
	    tcosfix.medfile = medfile
	    tcosfix.limfile = limfile
	    tcosfix.verbose = vb
	    tcosfix.xbox = xbox
	    tcosfix.ybox = ybox
	    tcosfix.proc2 = yes
	    if (fill == "median")
		tcosfix.medrep = yes
	    else
		tcosfix.medrep = no
	    }
	imin1 = ""
	ipass = 0

# Compare images two at a time to remove cosmic rays
	list = templist
	while (fscan (list,imin1) != EOF) {

	# Make bad pixel file name
	    if (fill == "fixpix") {
		smatch (imin1, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin1, 1, lroot)
		    }
		else {
		    imroot = imin1
		    }
		imbp1 = imroot // "_bp.fits"
		}

	#  Save original file to Uncosmic/
	    if (moveold) {
		imr = "Uncosmic/" // imin1
		if (!access (imr))
		    copy (imin1, imr)
		}

	    if (imin2 != "") {
		if (ipass == 1) {

		#  Save second file to Uncosmic/
		    if (moveold) {
			imr = "Uncosmic/" // imin2
			if (!access (imr))
			    copy (imin2, imr)
			}

		# Mark cosmic rays and replace by interpolation
		    if (fill == "fixpix") {

		# Find the cosmic rays in these two images
			if (vb) {
			    print ("Marking cosmic rays in ",imin1," and ",imin2)
			    print ("to bad pixel files ",imbp1," and ",imbp2)
			    }
			if (!access (imbp1) || !access (imbp2)) {
			    if (access (imbp1))
				delete (imbp1)
			    if (access (imbp2))
				delete (imbp2)
			    tcosbad (imin1,imin2,imbp1,imbp2)
			    if (tcosbad.error > 0) {
				next
				}
			    imreplace (imbp1,3)
			    hedit (imbp1, "IMREP", "Cosmic rays expanded by 1.5 pixels")
			    imreplace (imbp2,3)
			    hedit (imbp2, "IMREP", "Cosmic rays expanded by 1.5 pixels")
			    }
			else {
			    print ("Bad pixel files ",imbp1," and ",imbp2," already exist.")
			    }

		    # Since FIXPIX was run by CCDPROC, move FIXPIX value to FIXPIX0
			imgetp (imin1, "FIXPIX")
			if (imgetp.value != "")
			    hedit (imin1, "FIXPIX0", imgetp.value, add+, update+, verify-)
			hedit (imin1, "FIXPIX", del+, verify-)
			imgetp (imin2, "FIXPIX")
			if (imgetp.value != "")
			    hedit (imin2, "FIXPIX0", imgetp.value, add+, update+, verify-)
			hedit (imin2, "FIXPIX", del+, verify-)

		    # Interpolate along columns across bad pixels
			if (vb) {
			    print ("Removing cosmic rays in ",imin1)
			    }
			fixpix (imin1, imbp1)
			if (vb) {
			    print ("Removing cosmic rays in ",imin2)
			    }
			fixpix (imin2, imbp2)

		    # Make history line including time
			gettime ()
			dstring = gettime.date // " " // gettime.time
			history = dstring//" cosmic rays replaced by FIXPIX"
			hedit (imin1, "TCOSMIC", history, add+, verify-)
			hedit (imin2, "TCOSMIC", history, add+, verify-)
			tcosbad.proc2 = no
			}

		# Replace cosmic rays from median or second image
		    else {
			if (vb) {
			    print ("Replacing cosmic rays in ",imin1)
			    }
			tcosfix (imin1,imin2)

			tcosfix.proc2 = no
			}
		    }

	    # After doing the first pair, operate only on the last-read image

	    # If interpolating
		else if (fill == "fixpix") {

		# Find the cosmic rays in the latest image
		    if (vb) {
			print ("Marking cosmic rays in ",imin1)
			}
		    if (!access (imbp1)) {
			tcosbad (imin1,imin2,imbp1,imbp2)
			imreplace (imbp1,3)
			hedit (imbp1, "IMREP", "Cosmic rays expanded by 1.5 pixels")
			}
		    else {
			print ("Bad pixel file ",imbp1," already exists.")
			}

		# Since we want to run CCDPROC again, delete FIXPIX keyword
		    imgetp (imin1, "FIXPIX")
		    if (imgetp.value != "")
			hedit (imin1, "FIXPIX0", imgetp.value, add+, verify-, update+)
		    hedit (imin1, "FIXPIX", del+, verify-)

		# Interpolate along columns across bad pixels
		    if (vb) {
			print ("Removing cosmic rays in ",imin1)
			}
		    fixpix (imin1, imbp1)

		# Make history line including time
		    gettime ()
		    dstring = gettime.date // " " // gettime.time
		    history = dstring // " cosmic rays removed"
		    hedit (imin1, "TCOSMIC", history, add+, verify-)
		    }

	    # If filling
		else {
		    if (vb) {
			print ("Replacing cosmic rays in ",imin1)
			}
		    tcosfix (imin1, imin2)
		    }
		}
	    if (fill == "fixpix") {
		if (movebp) {
		    imr = "Uncosmic/" // imbp2
		    if (access (imr))
			delete (imr)
		    rename (imbp2, imr)
		    }
		else if (dellist) {
		    delete (imbp2)
		    }
		imbp2 = imbp1
		}
	    imin2 = imin1
	    ipass = ipass + 1
	    }

# Save last bad pixel, median, and limit files to Uncosmic
	if (movebp) {
	    imr = "Uncosmic/" // medfile
	    if (access (imr))
		delete (imr)
	    rename (medfile, imr)
	    imr = "Uncosmic/" // limfile
	    if (access (imr))
		delete (imr)
	    rename (limfile, imr)
	    if (fill == "fixpix") {
		imr = "Uncosmic/" // imbp1
		if (access (imr))
		delete (imr)
		rename (imbp1, imr)
		}
	    }
	else if (dellist) {
	    delete (medfile)
	    if (fill == "fixpix") {
		delete (imbp2)
		delete (limfile)
		}
	    }
	if (dellist) {
	    delete (templist)
	    }
end
# Sep 24 2007	New task based on hectospec.hcosmic
# Oct  1 2007	Make better median and limit file names
# Oct  5 2007	Run tcosbad if either bad pixel file is missing
# Oct 10 2007	Add parameter savebp to save bad pixel masks in Uncosmic/
# Oct 29 2007	Add parameters to define size of statistics box
# Nov  8 2007	Add options to fill bad pixels from second image or median
# Nov  8 2007	Set threshold in tcoslim, not tcosbad and tcosfix
# Nov 14 2007	Write TCOSMIC keyword in tcosfix task
