# TEXTRACT -- Extract TRES image into one or two echelle files
# By Doug Mink, SAO Telescope Data Center
# January 17, 2008

procedure textract (images)

	file	images=""	{prompt="Input image(s) to extract"}
	file	aproot=""	{prompt="Root filename for aperature reference spectrum"}
	file	apdir="tresdata$"	{prompt="Directory for reference spectra"}
	bool	recenter=yes	{prompt="Recenter orders (yes or no)"}
	string	orders="1-51"	{prompt="Orders to read"}
	string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
	string	gain="1"	{prompt="Photon gain (photons/data number)"}
	real    npeaks=0.2	{prompt="Select brightest peaks"}
	bool	interactive=no	{prompt="Run apall interactively (y or n)"}
	bool	review=no	{prompt="Review results of extraction (y or n)"}
	bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
	bool	verbose=yes	{prompt="Log progress (y or n)"}
	bool	debug=no	{prompt="Print debugging information (y or n)"}

	begin
	bool vb, db
	file imin, imfib1, imfib2, imfib3, imroot, templist, listfile
	file outfib1, outfib2, outfib3, imref
	int lroot
	string history, ims, tchar, args

	ims = images
	vb = verbose
	db = debug

	if (!access ("database/"))
	    mkdir ("database")

# Get file names for aperture reference images
	if (aproot == "")
	    aproot = "flat"
	tapref.aproot = aproot
	tapref.apdir = apdir
	tapref.verbose = vb
	tapref.debug = db

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempexlist")
        if (tchar == '@') {
	    listfile = substr (ims,2,100)
	    copy (listfile, templist)
	    }
	else {
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make output file name
	    smatch (imin, ".ec")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		}
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    }
		else {
		    imroot = imin
		    }
		}
	    imin = imroot // ".fits"

# Make sure this is not already an extracted image
	    imgetp.default = "0"
	    imgetp (imin, "TEXTRACT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TEXTRACT: ", imin, " is already an extracted image")
		    print ("TEXTRACT = ", imgetp.value)
		    }
		next
		}

# Make output file names for the fibers which are exposed
	    trsfib (imin)
	    if (trsfib.fib1 != "OFF")
		outfib1 = imroot // "_fib1.ec.fits"
	    else
		outfib1 = ""
	    if (trsfib.fib2 != "OFF" && trsfib.fib3 == "OFF")
		outfib2 = imroot // "_fib2.ec.fits"
	    else
		outfib2 = ""
	    if (trsfib.fib3 != "OFF")
		outfib3 = imroot // "_fib3.ec.fits"
	    else
		outfib3 = ""
	    if (db) {
		if (outfib1 != "" && outfib2 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib1," and ", outfib2)
		else if (outfib1 != "" && outfib3 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib1," and ", outfib3)
		else if (outfib1 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib1)
		else if (outfib2 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib2)
		else if (outfib3 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib3)
		}

# Set up apall parameters

	    apall.apertures = orders
	    apall.format = "echelle"
	    apall.profiles = ""
	    apall.interactive = interactive
	    apall.find = no
	    apall.recenter = recenter
	    apall.resize = no
	    apall.edit = no
	    apall.trace = no
	    apall.fittrace = no
	    apall.extract = yes
	    apall.extras = no
	    apall.review = review
	    apall.line = INDEF
	    apall.nsum = 10
	    apall.lower = -5.
	    apall.upper = 5.
	    apall.apidtable = ""
	    apall.b_function = "legendre"
	    apall.b_order = 1
	    apall.b_sample = "-10:-6,6:10"
	    apall.b_naverage = -3
	    apall.b_niterate = 3
	    apall.b_low_reject = 3.
	    apall.b_high_reject = 3.
	    apall.b_grow = 0.
	    apall.width = 5.
	    apall.radius = 10.
	    apall.threshold = 0.
	    apall.minsep = 5.
	    apall.maxsep = 1000.
	    apall.order = "increasing"
	    apall.aprecenter = ""
	    apall.npeaks = 20
	    apall.shift = recenter
	    apall.llimit = INDEF
	    apall.ulimit = INDEF
	    apall.ylevel = 0.1
	    apall.peak = yes
	    apall.bkg = yes
	    apall.r_grow = 0.
	    apall.avglimits = no
	    apall.t_nsum = 10
	    apall.t_step = 10
	    apall.t_nlost = 3
	    apall.t_function = "legendre"
	    apall.t_order = 3
	    apall.t_sample = "*"
	    apall.t_naverage = 1
	    apall.t_niterate = 0
	    apall.t_low_reject = 3.
	    apall.t_high_reject = 3.
	    apall.t_grow = 0.
	    apall.background = "none"
	    apall.skybox = 1
	    apall.weights = "none"
	    apall.pfit = "fit1d"
	    apall.clean = no
	    apall.saturation = INDEF
	    apall.readnoise = "0."
	    apall.gain = "1."
	    apall.lsigma = 4.
	    apall.usigma = 4.
	    apall.nsubaps = 1

# Set time of operation
	    gettime ()
	    history = gettime.date // " " // gettime.time // " extracted from " // imin
	    hedit.show = vb

# Set up aperture reference(s) for this file
	    tapref (imin)

# Extract fiber 1 object (usually) spectrum to echelle file
	    if (outfib1 != "") {
		imref = aproot//trsfib.apsize//trsfib.binning//"1.fits"
		apall.references = imref
		print ("TEXTRACT: Extracting ",outfib1)
		apall (imin, output=outfib1)
		hedit (outfib1, "BANDID2", "echelle order", add+, verify-)
		hedit (outfib1, "TEXTRACT", history, add+, verify-)
		hedit (outfib1, "TEXAPREF", imref, add+, verify-)
		}

# Extract fiber 2 (usually) background spectrum to echelle file
	    if (outfib2 != "") {
		imref = aproot//trsfib.apsize//trsfib.binning//"2.fits"
		apall.references = imref
		print ("TEXTRACT: Extracting ",outfib2)
		apall (imin, output=outfib2)
		hedit (outfib2, "BANDID2", "echelle order", add+, verify-)
		hedit (outfib2, "TEXTRACT", history, add+, verify-)
		hedit (outfib2, "TEXAPREF", imref, add+, verify-)
		}

# Extract fiber 3 calibration spectrum to echelle file
	    if (outfib3 != "") {
		imref = aproot//trsfib.apsize//trsfib.binning//"3.fits"
		apall.references = imref
		print ("TEXTRACT: Extracting ",outfib3)
		apall (imin, output=outfib3)
		hedit (outfib3, "BANDID2", "echelle order", add+, verify-)
		hedit (outfib3, "TEXTRACT", history, add+, verify-)
		hedit (outfib3, "TEXAPREF", imref, add+, verify-)
		}
	    }

	if (deltemp)
	    delete (templist)

end

# Sep 13 2007	New task based on hectospec.hextract
# Sep 18 2007	Allow extraction of any of the three apertures of data
# Sep 21 2007	Use apall instead of apsum and recenter
# Sep 24 2007	Change aperture templates names to root||apsize||apnum
# Sep 25 2007	Add binning parameter
# Sep 25 2007	Change aperture template names to root||(b)||apsize||apnum
# Sep 25 2007	Add recenter parameter so extraction without shift is possible
# Oct  3 2007	Drop fibs, binning, and apsize parameters: read from files
# Oct  3 2007	Set up aperture reference images in tres.tapref
# Oct 11 2007	Do not use file names fro tapref; generate them here
# Dec 19 2007	Default data directory to tresdata$

# Jan 16 2008	Change tresfib to trsfib
# Jan 16 2008	Pass debug and verbose to tapref
